@@ -1,7 +1,7 @@
 /**
  * @file db.c sqlite backend
  * 
- * Copyright (C) 2007-2012  Lars Windolf <lars.lindner@gmail.com>
+ * Copyright (C) 2007-2012  Lars Windolf <lars.windolf@gmx.de>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -32,7 +32,7 @@
 #include "vfolder.h"
 
 /* You can find a schema description used by this version of Liferea at:
-   http://lzone.de/wiki/doku.php?id=liferea:v1.8:db_schema */
+   https://lzone.de/wiki/doku.php?id=liferea:v1.8:db_schema */
 
 static sqlite3	*db = NULL;
 gboolean searchFolderRebuild = FALSE;
