@@ -1,13 +1,13 @@
-/**
+/*
  * @file item_list_view.c  presenting items in a GtkTreeView
- *  
- * Copyright (C) 2004-2012 Lars Windolf <lars.lindner@gmail.com>
+ *
+ * Copyright (C) 2004-2015 Lars Windolf <lars.windolf@gmx.de>
  * Copyright (C) 2004-2006 Nathan J. Conrad <t98502@users.sourceforge.net>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version. 
+ * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -40,13 +40,14 @@
 #include "itemview.h"
 #include "newsbin.h"
 #include "social.h"
+#include "xml.h"
 #include "ui/browser_tabs.h"
 #include "ui/icons.h"
 #include "ui/liferea_shell.h"
 #include "ui/popup_menu.h"
 #include "ui/ui_common.h"
 
-/**
+/*
  * Important performance considerations: Early versions had performance problems
  * with the item list loading because of the following two problems:
  *
@@ -58,22 +59,22 @@
  * collections of feeds only by adding items to a new unattached tree store.
  */
 
-/** Enumeration of the columns in the itemstore. */
+/* Enumeration of the columns in the itemstore. */
 enum is_columns {
-	IS_TIME,		/**< Time of item creation */
-	IS_TIME_STR,		/**< Time of item creation as a string*/
-	IS_LABEL,		/**< Displayed name */
-	IS_STATEICON,		/**< Pixbuf reference to the item's state icon */
-	IS_NR,			/**< Item id, to lookup item ptr from parent feed */
-	IS_PARENT,		/**< Parent node pointer */
-	IS_FAVICON,		/**< Pixbuf reference to the item's feed's icon */
-	IS_ENCICON,		/**< Pixbuf reference to the item's enclosure icon */
-	IS_ENCLOSURE,		/**< Flag whether enclosure is attached or not */
-	IS_SOURCE,		/**< Source node pointer */
-	IS_STATE,		/**< Original item state (unread, flagged...) for sorting */
-	ITEMSTORE_UNREAD,	/**< Flag whether "unread" icon is to be shown */
-	ITEMSTORE_ALIGN,        /**< How to align title (RTL support) */
-	ITEMSTORE_LEN		/**< Number of columns in the itemstore */
+	IS_TIME,		/*<< Time of item creation */
+	IS_TIME_STR,		/*<< Time of item creation as a string*/
+	IS_LABEL,		/*<< Displayed name */
+	IS_STATEICON,		/*<< Pixbuf reference to the item's state icon */
+	IS_NR,			/*<< Item id, to lookup item ptr from parent feed */
+	IS_PARENT,		/*<< Parent node pointer */
+	IS_FAVICON,		/*<< Pixbuf reference to the item's feed's icon */
+	IS_ENCICON,		/*<< Pixbuf reference to the item's enclosure icon */
+	IS_ENCLOSURE,		/*<< Flag whether enclosure is attached or not */
+	IS_SOURCE,		/*<< Source node pointer */
+	IS_STATE,		/*<< Original item state (unread, flagged...) for sorting */
+	ITEMSTORE_WEIGHT,		/*<< Flag whether weight is to be bold and "unread" icon is to be shown */
+	ITEMSTORE_ALIGN,        /*<< How to align title (RTL support) */
+	ITEMSTORE_LEN		/*<< Number of columns in the itemstore */
 };
 
 typedef enum {
@@ -123,11 +124,17 @@ launch_item_selected (open_link_target_type open_link_target)
 
 struct ItemListViewPrivate {
 	GtkTreeView	*treeview;
-	
-	GHashTable 	*item_id_to_iter;	/**< hash table used for fast item id->tree iter lookup */
 
-	gboolean	batch_mode;		/**< TRUE if we are in batch adding mode */
-	GtkTreeStore	*batch_itemstore;	/**< GtkTreeStore prepared unattached and to be set on update() */
+	GSList		*item_ids;		/*<< list of all currently known item ids */
+
+	gboolean	batch_mode;		/*<< TRUE if we are in batch adding mode */
+	GtkTreeStore	*batch_itemstore;	/*<< GtkTreeStore prepared unattached and to be set on update() */
+
+	GtkTreeViewColumn	*enclosureColumn;
+	GtkTreeViewColumn	*faviconColumn;
+	GtkTreeViewColumn	*stateColumn;
+
+	gboolean	wideView;		/*<< TRUE if date has to be rendered into headline column (because date column is invisible) */
 };
 
 static GObjectClass *parent_class = NULL;
@@ -139,8 +146,7 @@ item_list_view_finalize (GObject *object)
 {
 	ItemListViewPrivate *priv = ITEM_LIST_VIEW_GET_PRIVATE (object);
 
-	if (priv->item_id_to_iter)
-		g_hash_table_destroy (priv->item_id_to_iter);
+	g_slist_free (priv->item_ids);
 	if (priv->batch_itemstore)
 		g_object_unref (priv->batch_itemstore);
 
@@ -161,32 +167,59 @@ item_list_view_class_init (ItemListViewClass *klass)
 
 /* helper functions for item <-> iter conversion */
 
-gboolean
-item_list_view_contains_id (ItemListView *ilv, gulong id)
-{
-	return (NULL != g_hash_table_lookup (ilv->priv->item_id_to_iter, GUINT_TO_POINTER (id)));
-}
-
 static gulong
 item_list_view_iter_to_id (ItemListView *ilv, GtkTreeIter *iter)
 {
 	gulong	id = 0;
-	
+
 	gtk_tree_model_get (gtk_tree_view_get_model (ilv->priv->treeview), iter, IS_NR, &id, -1);
 	return id;
 }
 
+gboolean
+item_list_view_contains_id (ItemListView *ilv, gulong id)
+{
+	return (NULL != g_slist_find (ilv->priv->item_ids, GUINT_TO_POINTER (id)));
+}
+
 static gboolean
 item_list_view_id_to_iter (ItemListView *ilv, gulong id, GtkTreeIter *iter)
 {
-	GtkTreeIter *old_iter;
+	gboolean        valid;
+	GtkTreeIter	old_iter;
+	GtkTreeModel    *model;
 
-	old_iter = g_hash_table_lookup (ilv->priv->item_id_to_iter, GUINT_TO_POINTER (id));
-	if (!old_iter)
-		return FALSE;
-	
-	*iter = *old_iter;
-	return TRUE;
+	/* Problem here is batch insertion using batch_itemstore
+	   so the item can be in the GtkTreeView attached store or
+	    in the batch_itemstore */
+
+	if (item_list_view_contains_id (ilv, id)) {
+		/* First search the tree view */
+		model = gtk_tree_view_get_model (ilv->priv->treeview);
+		valid = gtk_tree_model_get_iter_first (model, &old_iter);
+		while (valid) {
+		        gulong current_id = item_list_view_iter_to_id (ilv, &old_iter);
+	                if(current_id == id) {
+				*iter = old_iter;
+	                        return TRUE;
+			}
+	                valid = gtk_tree_model_iter_next (model, &old_iter);
+		}
+
+		/* Next search the batch store */
+		model = GTK_TREE_MODEL (ilv->priv->batch_itemstore);
+		valid = gtk_tree_model_get_iter_first (model, &old_iter);
+		while (valid) {
+			gulong current_id;
+			gtk_tree_model_get (model, &old_iter, IS_NR, &current_id, -1);
+	                if(current_id == id) {
+				*iter = old_iter;
+	                        return TRUE;
+			}
+	                valid = gtk_tree_model_iter_next (model, &old_iter);
+		}
+	}
+	return FALSE;
 }
 
 static gint
@@ -198,13 +231,13 @@ item_list_view_date_sort_func (GtkTreeModel *model, GtkTreeIter *a, GtkTreeIter
 	gtk_tree_model_get (model, a, IS_TIME, &timea, -1);
 	gtk_tree_model_get (model, b, IS_TIME, &timeb, -1);
 	diff = difftime ((time_t)timeb, (time_t)timea);
-	
+
 	if (diff < 0)
 		return 1;
-		
+
 	if (diff > 0)
 		return -1;
-	
+
 	return 0;
 }
 
@@ -212,13 +245,13 @@ static gint
 item_list_view_favicon_sort_func (GtkTreeModel *model, GtkTreeIter *a, GtkTreeIter *b, gpointer user_data)
 {
 	nodePtr	node1, node2;
-	
+
 	gtk_tree_model_get (model, a, IS_SOURCE, &node1, -1);
 	gtk_tree_model_get (model, b, IS_SOURCE, &node2, -1);
-	
+
 	if (!node1->id || !node2->id)
 		return 0;
-		
+
 	return strcmp (node1->id, node2->id);
 }
 
@@ -226,29 +259,31 @@ void
 item_list_view_set_sort_column (ItemListView *ilv, nodeViewSortType sortType, gboolean sortReversed)
 {
 	gint sortColumn;
-	
+
 	switch (sortType) {
 		case NODE_VIEW_SORT_BY_TITLE:
-			sortColumn = IS_LABEL;
+			/* Some ugly switching here, because in wide view
+			   we do sort headlines by date */
+			if (ilv->priv->wideView)
+				sortColumn = IS_TIME;
+			else
+				sortColumn = IS_LABEL;
 			break;
 		case NODE_VIEW_SORT_BY_PARENT:
-			sortColumn = IS_PARENT;
-			break;
-		case NODE_VIEW_SORT_BY_STATE:
-			sortColumn = IS_STATE;
+			sortColumn = IS_SOURCE;
 			break;
 		case NODE_VIEW_SORT_BY_TIME:
 		default:
 			sortColumn = IS_TIME;
 			break;
 	}
-	
+
 	gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (gtk_tree_view_get_model (ilv->priv->treeview)),
-	                                      sortColumn, 
+	                                      sortColumn,
 	                                      sortReversed?GTK_SORT_DESCENDING:GTK_SORT_ASCENDING);
 }
 
-/**
+/*
  * Creates a GtkTreeStore to be filled with ui_itemlist_set_items
  * and to be set with ui_itemlist_set_tree_store().
  */
@@ -259,15 +294,15 @@ item_list_view_create_tree_store (void)
 	                    G_TYPE_UINT64,	/* IS_TIME */
 	                    G_TYPE_STRING, 	/* IS_TIME_STR */
 	                    G_TYPE_STRING,	/* IS_LABEL */
-	                    GDK_TYPE_PIXBUF,	/* IS_STATEICON */
+	                    G_TYPE_ICON,	/* IS_STATEICON */
 	                    G_TYPE_ULONG,	/* IS_NR */
 	                    G_TYPE_POINTER,	/* IS_PARENT */
-	                    GDK_TYPE_PIXBUF,	/* IS_FAVICON */
-	                    GDK_TYPE_PIXBUF,	/* IS_ENCICON */
+	                    G_TYPE_ICON,	/* IS_FAVICON */
+	                    G_TYPE_ICON,	/* IS_ENCICON */
 	                    G_TYPE_BOOLEAN,	/* IS_ENCLOSURE */
 	                    G_TYPE_POINTER,	/* IS_SOURCE */
 	                    G_TYPE_UINT,	/* IS_STATE */
-			    G_TYPE_INT,		/* ITEMSTORE_UNREAD */
+			    G_TYPE_INT,		/* ITEMSTORE_WEIGHT */
 			    G_TYPE_FLOAT        /* ITEMSTORE_ALIGN */
 	);
 }
@@ -279,12 +314,17 @@ on_itemlist_selection_changed (GtkTreeSelection *selection, gpointer user_data)
 	GtkTreeModel	*model;
 	itemPtr		item = NULL;
 
-	if (gtk_tree_selection_get_selected (selection, &model, &iter))
-		item = item_load (item_list_view_iter_to_id (ITEM_LIST_VIEW (user_data), &iter));
-
-	liferea_shell_update_item_menu (NULL != item);
-	if (item)
-		itemlist_selection_changed (item);
+	if (gtk_tree_selection_get_selected (selection, &model, &iter)) {
+		gulong id = item_list_view_iter_to_id (ITEM_LIST_VIEW (user_data), &iter);
+		if (id != itemlist_get_selected_id ()) {
+			item = item_load (id);
+			liferea_shell_update_item_menu (NULL != item);
+			if (item)
+				itemlist_selection_changed (item);
+		}
+	} else {
+		liferea_shell_update_item_menu (FALSE);
+	}
 }
 
 static void
@@ -293,14 +333,14 @@ itemlist_sort_column_changed_cb (GtkTreeSortable *treesortable, gpointer user_da
 	gint		sortColumn, nodeSort;
 	GtkSortType	sortType;
 	gboolean	sorted, changed;
-	
+
 	if (feedlist_get_selected () == NULL)
 		return;
-	
+
 	sorted = gtk_tree_sortable_get_sort_column_id (treesortable, &sortColumn, &sortType);
 	if (!sorted)
 		return;
-		
+
 	switch (sortColumn) {
 		case IS_TIME:
 		default:
@@ -310,8 +350,6 @@ itemlist_sort_column_changed_cb (GtkTreeSortable *treesortable, gpointer user_da
 			nodeSort = NODE_VIEW_SORT_BY_TITLE;
 			break;
 		case IS_STATE:
-			nodeSort = NODE_VIEW_SORT_BY_STATE;
-			break;
 		case IS_PARENT:
 		case IS_SOURCE:
 			nodeSort = NODE_VIEW_SORT_BY_PARENT;
@@ -323,7 +361,7 @@ itemlist_sort_column_changed_cb (GtkTreeSortable *treesortable, gpointer user_da
 		feedlist_schedule_save ();
 }
 
-/**
+/*
  * Sets a GtkTreeView to the active GtkTreeView.
  */
 static void
@@ -337,11 +375,11 @@ item_list_view_set_tree_store (ItemListView *ilv, GtkTreeStore *itemstore)
 	gtk_tree_view_set_model (ilv->priv->treeview, NULL);
 	if (model)
 		g_object_unref (model);
-	
+
 	gtk_tree_sortable_set_sort_func (GTK_TREE_SORTABLE (itemstore), IS_TIME, item_list_view_date_sort_func, NULL, NULL);
 	gtk_tree_sortable_set_sort_func (GTK_TREE_SORTABLE (itemstore), IS_SOURCE, item_list_view_favicon_sort_func, NULL, NULL);
 	g_signal_connect (G_OBJECT (itemstore), "sort-column-changed", G_CALLBACK (itemlist_sort_column_changed_cb), NULL);
-	
+
 	gtk_tree_view_set_model (ilv->priv->treeview, GTK_TREE_MODEL (itemstore));
 
 	/* Setup the selection handler */
@@ -354,22 +392,22 @@ item_list_view_set_tree_store (ItemListView *ilv, GtkTreeStore *itemstore)
 void
 item_list_view_remove_item (ItemListView *ilv, itemPtr item)
 {
-	GtkTreeIter	*iter;
+	GtkTreeIter	iter;
 
 	g_assert (NULL != item);
-	iter = g_hash_table_lookup (ilv->priv->item_id_to_iter, GUINT_TO_POINTER (item->id));
-	if (iter) {
+	if (item_list_view_id_to_iter (ilv, item->id, &iter)) {
 		/* Using the GtkTreeIter check if it is currently selected. If yes,
 		   scroll down by one in the sorted GtkTreeView to ensure something
 		   is selected after removing the GtkTreeIter */
-		if (gtk_tree_selection_iter_is_selected (gtk_tree_view_get_selection (ilv->priv->treeview), iter))
+		if (gtk_tree_selection_iter_is_selected (gtk_tree_view_get_selection (ilv->priv->treeview), &iter))
 			ui_common_treeview_move_cursor (ilv->priv->treeview, 1);
-	
-		gtk_tree_store_remove (GTK_TREE_STORE (gtk_tree_view_get_model (ilv->priv->treeview)), iter);
-		g_hash_table_remove (ilv->priv->item_id_to_iter, GUINT_TO_POINTER (item->id));
+
+		gtk_tree_store_remove (GTK_TREE_STORE (gtk_tree_view_get_model (ilv->priv->treeview)), &iter);
 	} else {
-		g_warning ("Fatal: item to be removed not found in iter lookup hash!");
+		g_warning ("Fatal: item to be removed not found in item id list!");
 	}
+
+	ilv->priv->item_ids = g_slist_remove (ilv->priv->item_ids, GUINT_TO_POINTER (item->id));
 }
 
 /* cleans up the item list, sets up the iter hash when called for the first time */
@@ -386,7 +424,7 @@ item_list_view_clear (ItemListView *ilv)
 	   whose removal is deferred until unselecting */
 	select = gtk_tree_view_get_selection (ilv->priv->treeview);
 	gtk_tree_selection_unselect_all (select);
-	
+
 	adj = gtk_scrollable_get_vadjustment (GTK_SCROLLABLE (ilv->priv->treeview));
 	gtk_adjustment_set_value (adj, 0.0);
 	gtk_scrollable_set_vadjustment (GTK_SCROLLABLE (ilv->priv->treeview), adj);
@@ -396,11 +434,11 @@ item_list_view_clear (ItemListView *ilv)
 
 	if (itemstore)
 		gtk_tree_store_clear (itemstore);
-	if (ilv->priv->item_id_to_iter)
-		g_hash_table_destroy (ilv->priv->item_id_to_iter);
-	
-	ilv->priv->item_id_to_iter = g_hash_table_new_full (g_direct_hash, g_direct_equal, NULL, g_free);
-	
+	if (ilv->priv->item_ids)
+		g_slist_free (ilv->priv->item_ids);
+
+	ilv->priv->item_ids = NULL;
+
 	/* enable batch mode for following item adds */
 	ilv->priv->batch_mode = TRUE;
 	ilv->priv->batch_itemstore = item_list_view_create_tree_store ();
@@ -431,15 +469,32 @@ item_list_view_update_item (ItemListView *ilv, itemPtr item)
 	GtkTreeStore	*itemstore;
 	GtkTreeIter	iter;
 	gchar		*title, *time_str;
-	const GdkPixbuf	*state_icon;
-	
+	const GIcon	*state_icon;
+
 	if (!item_list_view_id_to_iter (ilv, item->id, &iter))
 		return;
-	
+
 	time_str = (0 != item->time) ? date_format ((time_t)item->time, NULL) : g_strdup ("");
 
 	title = item->title && strlen (item->title) ? item->title : _("*** No title ***");
-	title = g_strstrip (g_strdup (title));
+	title = g_strstrip (g_markup_escape_text (title, -1));
+
+	if (ilv->priv->wideView) {
+		const gchar *important = _(" <span background='red' color='black'> important </span> ");
+		gchar *teaser = item_get_teaser (item);
+		gchar *tmp = title;
+
+		title = g_strdup_printf ("<span weight='%s' size='larger'>%s</span>%s\n<span weight='%s'>%s%s</span><span size='smaller' weight='ultralight'> — %s</span>",
+		                         item->readStatus?"normal":"ultrabold",
+		                         title,
+		                         item->flagStatus?important:"",
+		                         item->readStatus?"ultralight":"light",
+		                         teaser?teaser:"",
+		                         teaser?"…":"",
+					 time_str);
+		g_free (tmp);
+		g_free (teaser);
+	}
 
 	state_icon = item->flagStatus ? icon_get (ICON_FLAG) :
 	             !item->readStatus ? icon_get (ICON_UNREAD) :
@@ -449,13 +504,15 @@ item_list_view_update_item (ItemListView *ilv, itemPtr item)
 		itemstore = ilv->priv->batch_itemstore;
 	else
 		itemstore = GTK_TREE_STORE (gtk_tree_view_get_model (ilv->priv->treeview));
+
 	gtk_tree_store_set (itemstore,
 	                    &iter,
 		            IS_LABEL, title,
+	                    IS_TIME, item->time,
 			    IS_TIME_STR, time_str,
 			    IS_STATEICON, state_icon,
-			    ITEMSTORE_UNREAD, item->readStatus ? PANGO_WEIGHT_NORMAL : PANGO_WEIGHT_BOLD,
 			    ITEMSTORE_ALIGN, item_list_title_alignment (title),
+	                    ITEMSTORE_WEIGHT, item->readStatus ? PANGO_WEIGHT_NORMAL : PANGO_WEIGHT_BOLD,
 			    -1);
 
 	g_free (time_str);
@@ -468,27 +525,40 @@ item_list_view_update_item_foreach (gpointer key,
                                     gpointer user_data)
 {
 	itemPtr 	item;
-	
+
 	item = item_load (GPOINTER_TO_UINT (key) /* id */);
 	if (!item)
 		return;
 
 	item_list_view_update_item (ITEM_LIST_VIEW (user_data), item);
-	
+
 	item_unload (item);
 }
 
-void 
-item_list_view_update_all_items (ItemListView *ilv) 
+void
+item_list_view_update_all_items (ItemListView *ilv)
 {
-	g_hash_table_foreach (ilv->priv->item_id_to_iter, item_list_view_update_item_foreach, (gpointer)ilv);
+        gboolean        valid;
+        GtkTreeIter     iter;
+        gulong          id;
+	GtkTreeModel    *model;
+
+	model = gtk_tree_view_get_model (ilv->priv->treeview);
+        valid = gtk_tree_model_get_iter_first (model, &iter);
+	while (valid) {
+		gtk_tree_model_get (model, &iter, IS_NR, &id, -1);
+                itemPtr	item = item_load (id);
+                item_list_view_update_item (ilv, item);
+                item_unload (item);
+                valid = gtk_tree_model_iter_next (model, &iter);
+	}
 }
 
 void
 item_list_view_update (ItemListView *ilv, gboolean hasEnclosures)
 {
 	/* we depend on the fact that the third column is the enclosure icon column!!! */
-	gtk_tree_view_column_set_visible (gtk_tree_view_get_column (ilv->priv->treeview, 1), hasEnclosures);
+	gtk_tree_view_column_set_visible (ilv->priv->enclosureColumn, hasEnclosures);
 
 	if (ilv->priv->batch_mode) {
 		item_list_view_set_tree_store (ilv, ilv->priv->batch_itemstore);
@@ -500,7 +570,7 @@ item_list_view_update (ItemListView *ilv, gboolean hasEnclosures)
 }
 
 static gboolean
-on_item_list_view_key_press_event (GtkWidget *widget, GdkEventKey *event, gpointer data) 
+on_item_list_view_key_press_event (GtkWidget *widget, GdkEventKey *event, gpointer data)
 {
 	if ((event->type == GDK_KEY_PRESS) && (event->state == 0)
 	    && (event->keyval == GDK_KEY_Delete))
@@ -530,7 +600,7 @@ get_cell_renderer_width (GtkWidget *widget, GtkCellRenderer *cell, const gchar *
 }
 
 static gboolean
-on_item_list_view_query_tooltip (GtkWidget *widget, gint x, gint y, gboolean keyboard_mode, GtkTooltip *tooltip, GtkTreeViewColumn *headline_column) 
+on_item_list_view_query_tooltip (GtkWidget *widget, gint x, gint y, gboolean keyboard_mode, GtkTooltip *tooltip, GtkTreeViewColumn *headline_column)
 {
 	GtkTreeView *view = GTK_TREE_VIEW (widget);
 	GtkTreeModel *model; GtkTreePath *path; GtkTreeIter iter;
@@ -550,7 +620,7 @@ on_item_list_view_query_tooltip (GtkWidget *widget, gint x, gint y, gboolean key
 
 			gchar *text;
 			gint weight;
-			gtk_tree_model_get (model, &iter, IS_LABEL, &text, ITEMSTORE_UNREAD, &weight, -1);
+			gtk_tree_model_get (model, &iter, IS_LABEL, &text, ITEMSTORE_WEIGHT, &weight, -1);
 
 			gint full_width = get_cell_renderer_width (widget, cell, text, weight);
 			gint column_width = gtk_tree_view_column_get_width (column);
@@ -573,10 +643,9 @@ on_item_list_view_button_press_event (GtkWidget *treeview, GdkEventButton *event
 	ItemListView		*ilv = ITEM_LIST_VIEW (user_data);
 	GtkTreePath		*path;
 	GtkTreeIter		iter;
-	GtkTreeViewColumn	*column;
 	itemPtr			item = NULL;
 	gboolean		result = FALSE;
-	
+
 	if (event->type != GDK_BUTTON_PRESS)
 		return FALSE;
 
@@ -589,18 +658,25 @@ on_item_list_view_button_press_event (GtkWidget *treeview, GdkEventButton *event
 
 	if (gtk_tree_model_get_iter (gtk_tree_view_get_model (ilv->priv->treeview), &iter, path))
 		item = item_load (item_list_view_iter_to_id (ilv, &iter));
-		
+
 	gtk_tree_path_free (path);
-	
+
 	if (item) {
-		GdkEventButton *eb = (GdkEventButton*)event; 
+		GdkEventButton *eb = (GdkEventButton*)event;
 		switch (eb->button) {
 			case 1:
-				column = gtk_tree_view_get_column (ilv->priv->treeview, 0);
-				if (column) {
-					/* Allow flag toggling when left clicking in the flagging column.
-					   We depend on the fact that the state column is the first!!! */
-					if (event->x <= gtk_tree_view_column_get_width (column)) {
+				/* Allow flag toggling when left clicking in the
+				   state or favicon column. We depend
+				   on the fact that those columns are all left
+				   of the headline column !!! */
+				if(gtk_tree_view_column_get_visible (ilv->priv->faviconColumn)){
+					if (event->x <= (gtk_tree_view_column_get_width (ilv->priv->faviconColumn))) {
+						itemlist_toggle_flag (item);
+						result = TRUE;
+					}
+				}
+				else{
+					if (event->x <= (gtk_tree_view_column_get_width (ilv->priv->stateColumn))) {
 						itemlist_toggle_flag (item);
 						result = TRUE;
 					}
@@ -622,7 +698,7 @@ on_item_list_view_button_press_event (GtkWidget *treeview, GdkEventButton *event
 		}
 		item_unload (item);
 	}
-		
+
 	return result;
 }
 
@@ -647,7 +723,7 @@ static void
 on_Itemlist_row_activated (GtkTreeView *treeview,
                            GtkTreePath *path,
 			   GtkTreeViewColumn *column,
-			   gpointer user_data) 
+			   gpointer user_data)
 {
     launch_item_selected (DEFAULT);
 }
@@ -662,11 +738,10 @@ static void
 item_list_view_init (ItemListView *ilv)
 {
 	ilv->priv = ITEM_LIST_VIEW_GET_PRIVATE (ilv);
-	ilv->priv->item_id_to_iter = g_hash_table_new_full (g_direct_hash, g_direct_equal, NULL, g_free);
 }
 
 ItemListView *
-item_list_view_create (GtkWidget *window) 
+item_list_view_create (gboolean wide)
 {
 	ItemListView		*ilv;
 	GtkCellRenderer		*renderer;
@@ -674,55 +749,79 @@ item_list_view_create (GtkWidget *window)
 	GtkWidget 		*ilscrolledwindow;
 
 	ilv = g_object_new (ITEM_LIST_VIEW_TYPE, NULL);
-		
+	ilv->priv->wideView = wide;
+
 	ilscrolledwindow = gtk_scrolled_window_new (NULL, NULL);
 	gtk_widget_show (ilscrolledwindow);
-	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (ilscrolledwindow), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
+
+	if (wide)
+		gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (ilscrolledwindow), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
+	else
+		gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (ilscrolledwindow), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
 	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (ilscrolledwindow), GTK_SHADOW_IN);
 
 	ilv->priv->treeview = GTK_TREE_VIEW (gtk_tree_view_new ());
+	if (wide) {
+		gtk_tree_view_set_fixed_height_mode (ilv->priv->treeview, FALSE);
+		gtk_tree_view_set_grid_lines (ilv->priv->treeview, GTK_TREE_VIEW_GRID_LINES_HORIZONTAL);
+	}
 	gtk_container_add (GTK_CONTAINER (ilscrolledwindow), GTK_WIDGET (ilv->priv->treeview));
 	gtk_widget_show (GTK_WIDGET (ilv->priv->treeview));
 	gtk_widget_set_name (GTK_WIDGET (ilv->priv->treeview), "itemlist");
-	gtk_tree_view_set_rules_hint (ilv->priv->treeview, TRUE);
-	
-	g_object_set_data (G_OBJECT (window), "itemlist", ilv->priv->treeview);
 
 	item_list_view_set_tree_store (ilv, item_list_view_create_tree_store ());
 
 	renderer = gtk_cell_renderer_pixbuf_new ();
-	column = gtk_tree_view_column_new_with_attributes ("", renderer, "pixbuf", IS_STATEICON, NULL);
-	gtk_tree_view_append_column (ilv->priv->treeview, column);
-	gtk_tree_view_column_set_sort_column_id (column, IS_STATE);	
-	
-	renderer = gtk_cell_renderer_pixbuf_new ();
-	column = gtk_tree_view_column_new_with_attributes ("", renderer, "pixbuf", IS_ENCICON, NULL);
+	column = gtk_tree_view_column_new_with_attributes ("", renderer, "gicon", IS_STATEICON, NULL);
+	g_object_set (renderer, "stock-size", wide?GTK_ICON_SIZE_LARGE_TOOLBAR:GTK_ICON_SIZE_SMALL_TOOLBAR, NULL);
 	gtk_tree_view_append_column (ilv->priv->treeview, column);
+	ilv->priv->stateColumn = column;
+	gtk_tree_view_column_set_sort_column_id (column, IS_STATE);
+	if (wide)
+		gtk_tree_view_column_set_visible (column, FALSE);
 
-	renderer = gtk_cell_renderer_text_new ();
-	column = gtk_tree_view_column_new_with_attributes (_("Date"), renderer, 
-	                                                   "text", IS_TIME_STR,
-							   "weight", ITEMSTORE_UNREAD,
-							   NULL);
-	gtk_tree_view_append_column (ilv->priv->treeview, column);
-	gtk_tree_view_column_set_sort_column_id(column, IS_TIME);
-	g_object_set (column, "resizable", TRUE, NULL);
-	
 	renderer = gtk_cell_renderer_pixbuf_new ();
-	column = gtk_tree_view_column_new_with_attributes ("", renderer, "pixbuf", IS_FAVICON, NULL);
+	column = gtk_tree_view_column_new_with_attributes ("", renderer, "gicon", IS_FAVICON, NULL);
+	g_object_set (renderer, "stock-size", wide?GTK_ICON_SIZE_DIALOG:GTK_ICON_SIZE_SMALL_TOOLBAR, NULL);
+
 	gtk_tree_view_column_set_sort_column_id (column, IS_SOURCE);
 	gtk_tree_view_append_column (ilv->priv->treeview, column);
-	
+	ilv->priv->faviconColumn = column;
+
+
 	renderer = gtk_cell_renderer_text_new ();
-	headline_column = gtk_tree_view_column_new_with_attributes (_("Headline"), renderer, 
-	                                                   "text", IS_LABEL,
-							   "weight", ITEMSTORE_UNREAD,
+	headline_column = gtk_tree_view_column_new_with_attributes (_("Headline"), renderer,
+	                                                   "markup", IS_LABEL,
 							   "xalign", ITEMSTORE_ALIGN,
 							   NULL);
+	gtk_tree_view_column_set_expand (headline_column, TRUE);
 	gtk_tree_view_append_column (ilv->priv->treeview, headline_column);
-	gtk_tree_view_column_set_sort_column_id (headline_column, IS_LABEL);
 	g_object_set (headline_column, "resizable", TRUE, NULL);
-	g_object_set (renderer, "ellipsize", PANGO_ELLIPSIZE_END, NULL);
+	if (wide) {
+		gtk_tree_view_column_set_sort_column_id (headline_column, IS_TIME);
+		g_object_set (renderer, "wrap-mode", PANGO_WRAP_WORD, NULL);
+		g_object_set (renderer, "wrap-width", 300, NULL);
+	} else {
+		gtk_tree_view_column_set_sort_column_id (headline_column, IS_LABEL);
+		g_object_set (renderer, "ellipsize", PANGO_ELLIPSIZE_END, NULL);
+		gtk_tree_view_column_add_attribute (headline_column, renderer, "weight", ITEMSTORE_WEIGHT);
+	}
+
+	renderer = gtk_cell_renderer_pixbuf_new ();
+	column = gtk_tree_view_column_new_with_attributes ("", renderer, "gicon", IS_ENCICON, NULL);
+	gtk_tree_view_append_column (ilv->priv->treeview, column);
+	ilv->priv->enclosureColumn = column;
+
+	renderer = gtk_cell_renderer_text_new ();
+	column = gtk_tree_view_column_new_with_attributes (_("Date"), renderer,
+		                                           "text", IS_TIME_STR,
+	                                                   "weight", ITEMSTORE_WEIGHT,
+							   NULL);
+	gtk_tree_view_column_set_sizing(column, GTK_TREE_VIEW_COLUMN_GROW_ONLY);
+	gtk_tree_view_append_column (ilv->priv->treeview, column);
+	gtk_tree_view_column_set_sort_column_id(column, IS_TIME);
+	if (wide)
+		gtk_tree_view_column_set_visible (column, FALSE);
 
 	/* And connect signals */
 	g_signal_connect (G_OBJECT (ilv->priv->treeview), "button_press_event", G_CALLBACK (on_item_list_view_button_press_event), ilv);
@@ -730,9 +829,11 @@ item_list_view_create (GtkWidget *window)
 	g_signal_connect (G_OBJECT (ilv->priv->treeview), "key-press-event", G_CALLBACK (on_item_list_view_key_press_event), ilv);
 	g_signal_connect (G_OBJECT (ilv->priv->treeview), "popup_menu", G_CALLBACK (on_item_list_view_popup_menu), ilv);
 
-	gtk_widget_set_has_tooltip (GTK_WIDGET (ilv->priv->treeview), TRUE);
-	g_signal_connect (G_OBJECT (ilv->priv->treeview), "query-tooltip", G_CALLBACK (on_item_list_view_query_tooltip), headline_column);
-		  
+	if (!wide) {
+		gtk_widget_set_has_tooltip (GTK_WIDGET (ilv->priv->treeview), TRUE);
+		g_signal_connect (G_OBJECT (ilv->priv->treeview), "query-tooltip", G_CALLBACK (on_item_list_view_query_tooltip), headline_column);
+	}
+
 	return ilv;
 }
 
@@ -744,39 +845,39 @@ item_list_view_add_item_to_tree_store (ItemListView *ilv, GtkTreeStore *itemstor
 	GtkTreeIter	*iter;
 	GtkTreeIter	old_iter;
 	gboolean	exists;
-		
+
 	if (item->flagStatus)
 		state += 2;
 	if (!item->readStatus)
 		state += 1;
-		
+
 	node = node_from_id (item->nodeId);
 	if(!node)
 		return;	/* comment items do cause this... maybe filtering them earlier would be a good idea... */
-		
+
 	exists = item_list_view_id_to_iter (ilv, item->id, &old_iter);
 	iter = &old_iter;
-	
-	if (!exists) 
+
+	if (!exists)
 	{
 		iter = g_new0 (GtkTreeIter, 1);
 		gtk_tree_store_prepend (itemstore, iter, NULL);
-		g_hash_table_insert (ilv->priv->item_id_to_iter, GUINT_TO_POINTER (item->id), (gpointer)iter);
+		ilv->priv->item_ids = g_slist_append (ilv->priv->item_ids, GUINT_TO_POINTER (item->id));
 	}
 
 	gtk_tree_store_set (itemstore, iter,
 		                       IS_TIME, (guint64)item->time,
 		                       IS_NR, item->id,
 				       IS_PARENT, node,
-		                       IS_FAVICON, node->icon,
+		                       IS_FAVICON, node_get_icon (node),
 		                       IS_ENCICON, item->hasEnclosure?icon_get (ICON_ENCLOSURE):NULL,
 				       IS_ENCLOSURE, item->hasEnclosure,
 				       IS_SOURCE, node,
 				       IS_STATE, state,
-		                       -1);		                       
+		                       -1);
 }
 
-void 
+void
 item_list_view_add_item (ItemListView *ilv, itemPtr item)
 {
 	GtkTreeStore	*itemstore;
@@ -785,45 +886,46 @@ item_list_view_add_item (ItemListView *ilv, itemPtr item)
 		/* either merge to new unattached GtkTreeStore */
 		item_list_view_add_item_to_tree_store (ilv, ilv->priv->batch_itemstore, item);
 	} else {
-		/* or merge to visible item store */	
+		/* or merge to visible item store */
 		itemstore = GTK_TREE_STORE (gtk_tree_view_get_model (ilv->priv->treeview));
 		item_list_view_add_item_to_tree_store (ilv, itemstore, item);
 	}
-	
+
 	item_list_view_update_item (ilv, item);
 }
 
 void
 item_list_view_enable_favicon_column (ItemListView *ilv, gboolean enabled)
 {
-	/* we depend on the fact that the second column is the favicon column!!! 
-	   if we are in search mode or have a folder or vfolder we show the favicon 
-	   column to give a hint where the item comes from ... */
-	gtk_tree_view_column_set_visible (gtk_tree_view_get_column (ilv->priv->treeview, 3), enabled);
+	gtk_tree_view_column_set_visible (ilv->priv->faviconColumn, enabled);
+
+	// In wide view we want to save vertical space and hide the state column
+	if (ilv->priv->wideView)
+		gtk_tree_view_column_set_visible (ilv->priv->stateColumn, !enabled);
 }
 
 void
-on_popup_launch_item_selected (void) 
+on_popup_launch_item_selected (void)
 {
-    launch_item_selected (INTERNAL);
+	launch_item_selected (INTERNAL);
 }
 
 void
-on_popup_launch_item_in_tab_selected (void) 
+on_popup_launch_item_in_tab_selected (void)
 {
-    launch_item_selected (TAB);
+	launch_item_selected (TAB);
 }
 
 void
-on_popup_launch_item_external_selected (void) 
+on_popup_launch_item_external_selected (void)
 {
-    launch_item_selected (EXTERNAL);
+	launch_item_selected (EXTERNAL);
 }
 
 /* menu callbacks */
 
 void
-on_toggle_item_flag (GtkMenuItem *menuitem, gpointer user_data) 
+on_toggle_item_flag (GtkMenuItem *menuitem, gpointer user_data)
 {
 	itemPtr		item;
 
@@ -834,14 +936,14 @@ on_toggle_item_flag (GtkMenuItem *menuitem, gpointer user_data)
 	}
 }
 
-void 
+void
 on_popup_toggle_flag (void)
 {
 	on_toggle_item_flag (NULL, NULL);
 }
 
-void 
-on_toggle_unread_status (GtkMenuItem *menuitem, gpointer user_data) 
+void
+on_toggle_unread_status (GtkMenuItem *menuitem, gpointer user_data)
 {
 	itemPtr		item;
 
@@ -862,7 +964,7 @@ void
 on_remove_items_activate (GtkMenuItem *menuitem, gpointer user_data)
 {
 	nodePtr		node;
-	
+
 	node = feedlist_get_selected ();
 	// FIXME: use node type capability check
 	if (node && (IS_FEED (node) || IS_NEWSBIN (node)))
@@ -875,7 +977,7 @@ void
 on_remove_item_activate (GtkMenuItem *menuitem, gpointer user_data)
 {
 	itemPtr		item;
-	
+
 	item = itemlist_get_selected ();
 	if (item) {
 		itemview_select_item (NULL);
@@ -896,18 +998,12 @@ item_list_view_select (ItemListView *ilv, itemPtr item)
 {
 	GtkTreeView		*treeview = ilv->priv->treeview;
 	GtkTreeSelection	*selection;
-	
+	GtkTreeIter		iter;
+
 	selection = gtk_tree_view_get_selection (treeview);
-	
-	if (item) {
-		GtkTreeIter		iter;
-		GtkTreePath		*path;
-		
-		if (!item_list_view_id_to_iter(ilv, item->id, &iter))
-			/* This is an evil hack to fix SF #1870052: crash
-			   upon hitting <enter> when no headline selected.
-			   FIXME: This code is rotten! Rewrite it! Now! */
-			itemlist_selection_changed (NULL);
+
+	if (item && item_list_view_id_to_iter(ilv, item->id, &iter)){
+		GtkTreePath	*path = NULL;
 
 		path = gtk_tree_model_get_path (gtk_tree_view_get_model (treeview), &iter);
 		if (path) {
@@ -916,6 +1012,8 @@ item_list_view_select (ItemListView *ilv, itemPtr item)
 			gtk_tree_path_free (path);
 		}
 	} else {
+		if (item)
+			g_warning ("item_list_view_select : attempt to select an item which is not present in the view.");
 		gtk_tree_selection_unselect_all (selection);
 	}
 }
@@ -926,14 +1024,14 @@ item_list_view_find_unread_item (ItemListView *ilv, gulong startId)
 	GtkTreeIter		iter;
 	GtkTreeModel		*model;
 	gboolean		valid = TRUE;
-	
+
 	model = gtk_tree_view_get_model (ilv->priv->treeview);
-	
+
 	if (startId)
 		valid = item_list_view_id_to_iter (ilv, startId, &iter);
 	else
 		valid = gtk_tree_model_get_iter_first (model, &iter);
-	
+
 	while (valid) {
 		itemPtr	item = item_load (item_list_view_iter_to_id (ilv, &iter));
 		if (item) {
