@@ -1,7 +1,7 @@
 /*
  * @file itemview.c  viewing feed content in different presentation modes
  * 
- * Copyright (C) 2006-2012 Lars Windolf <lars.lindner@gmail.com>
+ * Copyright (C) 2006-2015 Lars Windolf <lars.windolf@gmx.de>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -46,24 +46,30 @@
 #define ITEMVIEW_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), ITEMVIEW_TYPE, ItemViewPrivate))
 
 struct ItemViewPrivate {
-	gboolean	htmlOnly;		/**< TRUE if HTML only mode */
-	guint		mode;			/**< current item view mode */
-	nodePtr		node;			/**< the node whose items are displayed */
-	gboolean	browsing;		/**< TRUE if itemview is used as internal browser right now */
-	gboolean	needsHTMLViewUpdate;	/**< flag to be set when HTML rendering is to be 
+	gboolean	htmlOnly;		/*<< TRUE if HTML only mode */
+	guint		mode;			/*<< current item view mode */
+	nodePtr		node;			/*<< the node whose items are displayed */
+	gboolean	browsing;		/*<< TRUE if itemview is used as internal browser right now */
+	gboolean	needsHTMLViewUpdate;	/*<< flag to be set when HTML rendering is to be
 						     updated, used to delay HTML updates */
-	gboolean	hasEnclosures;		/**< TRUE if at least one item of the current itemset has an enclosure */
+	gboolean	hasEnclosures;		/*<< TRUE if at least one item of the current itemset has an enclosure */
 						     
-	nodeViewType	viewMode;		/**< current viewing mode */
-	guint		currentLayoutMode;	/**< layout mode (3 pane, 2 pane, wide view) */
+	nodeViewType	viewMode;		/*<< current viewing mode */
+	guint		currentLayoutMode;	/*<< layout mode (3 pane, 2 pane, wide view) */
 								     
-	GtkWidget	*itemListViewContainer;	/**< scrolled window holding item list tree view */
-	ItemListView	*itemListView;		/**< widget instance used to present items in list mode */
+	GtkWidget	*itemListViewContainer;	/*<< scrolled window holding item list tree view */
+	ItemListView	*itemListView;		/*<< widget instance used to present items in list mode */
 
-	EnclosureListView	*enclosureView;	/**< Enclosure list widget */
-	LifereaHtmlView		*htmlview;	/**< HTML rendering widget instance used to render single items and summaries mode */
+	EnclosureListView	*enclosureView;	/*<< Enclosure list widget */
+	LifereaHtmlView		*htmlview;	/*<< HTML rendering widget instance used to render single items and summaries mode */
 
-	gfloat			zoom;		/**< HTML rendering widget zoom level */
+	gfloat			zoom;		/*<< HTML rendering widget zoom level */
+};
+
+enum {
+	PROP_NONE,
+	PROP_ITEM_LIST_VIEW,
+	PROP_HTML_VIEW
 };
 
 static GObjectClass *parent_class = NULL;
@@ -91,6 +97,24 @@ itemview_finalize (GObject *object)
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
+static void
+itemview_get_property (GObject *object, guint prop_id, GValue *value, GParamSpec *pspec)
+{
+	ItemView *itemview = ITEMVIEW (object);
+
+	switch (prop_id) {
+		case PROP_ITEM_LIST_VIEW:
+			g_value_set_object (value, itemview->priv->itemListView);
+			break;
+		case PROP_HTML_VIEW:
+			g_value_set_object (value, itemview->priv->htmlview);
+			break;
+		default:
+			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+			break;
+	}
+}
+
 static void
 itemview_class_init (ItemViewClass *klass)
 {
@@ -98,8 +122,30 @@ itemview_class_init (ItemViewClass *klass)
 
 	parent_class = g_type_class_peek_parent (klass);
 
+	object_class->get_property = itemview_get_property;
+
 	object_class->finalize = itemview_finalize;
 
+	/* ItemView:item-list-view: */
+	g_object_class_install_property (object_class,
+					PROP_ITEM_LIST_VIEW,
+					g_param_spec_object (
+						"item-list-view",
+						"ItemListView",
+						"ItemListView object",
+						ITEM_LIST_VIEW_TYPE,
+						G_PARAM_READABLE));
+
+	/* ItemView:html-view: */
+	g_object_class_install_property (object_class,
+					PROP_HTML_VIEW,
+					g_param_spec_object (
+						"html-view",
+						"LifereaHtmlView",
+						"LifereaHtmlView object",
+						LIFEREA_HTMLVIEW_TYPE,
+						G_PARAM_READABLE));
+
 	g_type_class_add_private (object_class, sizeof(ItemViewPrivate));
 }
 
@@ -314,13 +360,7 @@ itemview_find_unread_item (gulong startId)
 void
 itemview_scroll (void)
 {
-	/* We try to scroll the HTML view, but if we are already at the
-	   bottom of the item view the scrolling will return FALSE and 
-	   we trigger Next-Unread to realize easy headline skimming. */
-	if (liferea_htmlview_scroll (itemview->priv->htmlview) == FALSE)
-		on_next_unread_item_activate (NULL, NULL);
-		
-	/* Note the above condition is duplicated in mozilla/mozsupport.cpp! */
+	liferea_htmlview_scroll (itemview->priv->htmlview);
 }
 
 void
@@ -360,6 +400,7 @@ void
 itemview_set_layout (nodeViewType newMode)
 {
 	ItemViewPrivate *ivp = itemview->priv;
+	GtkWidget 	*previous_parent = NULL;
 	const gchar	*htmlWidgetName, *ilWidgetName, *encViewVBoxName;
 	
 	if (newMode == ivp->currentLayoutMode)
@@ -369,16 +410,13 @@ itemview_set_layout (nodeViewType newMode)
 	liferea_shell_set_view_mode (newMode);
 	
 	if (!ivp->htmlview) {
-		GtkWidget *renderWidget;
-		
 		debug0 (DEBUG_GUI, "Creating HTML widget");
+		htmlview_init ();
 		ivp->htmlview = liferea_htmlview_new (FALSE);
+		liferea_htmlview_set_headline_view (ivp->htmlview);
 		g_signal_connect (ivp->htmlview, "statusbar-changed", 
 		                  G_CALLBACK (on_important_status_message), NULL);
-		renderWidget = liferea_htmlview_get_widget (ivp->htmlview);
-		gtk_container_add (GTK_CONTAINER (liferea_shell_lookup ("normalViewHtml")), renderWidget);
-		gtk_widget_show (renderWidget);
-		
+
 		/* Set initial zoom */
 		liferea_htmlview_set_zoom (itemview->priv->htmlview, itemview->priv->zoom/100.);
 	} else {	
@@ -410,10 +448,23 @@ itemview_set_layout (nodeViewType newMode)
 	}
 
 	/* Reparenting HTML view. This avoids the overhead of new browser instances. */
+	g_assert (htmlWidgetName);
 	gtk_notebook_set_current_page (GTK_NOTEBOOK (liferea_shell_lookup ("itemtabs")), newMode);
-	gtk_widget_reparent (liferea_htmlview_get_widget (ivp->htmlview), liferea_shell_lookup (htmlWidgetName));
-	if (ilWidgetName)
-		gtk_widget_reparent (GTK_WIDGET (ivp->itemListViewContainer), liferea_shell_lookup (ilWidgetName));
+	previous_parent = gtk_widget_get_parent (liferea_htmlview_get_widget (ivp->htmlview));
+	if (previous_parent)
+		gtk_container_remove (GTK_CONTAINER (previous_parent), liferea_htmlview_get_widget (ivp->htmlview));
+	gtk_container_add (GTK_CONTAINER (liferea_shell_lookup (htmlWidgetName)), liferea_htmlview_get_widget (ivp->htmlview));
+
+	/* Recreate the item list view */
+	if (ivp->itemListViewContainer)
+		gtk_widget_destroy (ivp->itemListViewContainer);
+
+	if (ilWidgetName) {
+		ivp->itemListView = item_list_view_create (newMode == NODE_VIEW_MODE_WIDE);
+		ivp->itemListViewContainer = gtk_widget_get_parent (GTK_WIDGET (item_list_view_get_widget (ivp->itemListView)));
+		g_assert (ivp->itemListViewContainer);
+		gtk_container_add (GTK_CONTAINER (liferea_shell_lookup (ilWidgetName)), GTK_WIDGET (ivp->itemListViewContainer));
+	}
 	
 	/* Destroy previous enclosure list. */
 	if (ivp->enclosureView) {
@@ -424,9 +475,9 @@ itemview_set_layout (nodeViewType newMode)
 	/* Create a new enclosure list GtkTreeView. */		
 	if (encViewVBoxName) {
 		ivp->enclosureView = enclosure_list_view_new ();
-		gtk_box_pack_end (GTK_BOX (liferea_shell_lookup (encViewVBoxName)),
+		gtk_grid_attach_next_to (GTK_GRID (liferea_shell_lookup (encViewVBoxName)),
 		                  enclosure_list_view_get_widget (ivp->enclosureView),
-				  FALSE, FALSE, 0);
+				  NULL, GTK_POS_BOTTOM, 1,1);
 		gtk_widget_show_all (liferea_shell_lookup (encViewVBoxName));
 	}
 }
@@ -436,27 +487,18 @@ itemview_create (GtkWidget *window)
 {
 	gint zoom;
 
-	/* 1. Create widgets, load preferences */
-	
 	g_object_new (ITEMVIEW_TYPE, NULL);
-	
-	itemview->priv->currentLayoutMode = NODE_VIEW_MODE_DEFAULT;
-	itemview->priv->itemListView = item_list_view_create (window);
-	itemview->priv->itemListViewContainer = gtk_widget_get_parent (GTK_WIDGET (item_list_view_get_widget (itemview->priv->itemListView)));
+
+	/* 1. Load preferences */
 	conf_get_int_value (LAST_ZOOMLEVEL, &zoom);
-	
 	if (zoom == 0) {
 		zoom = 100;
 		conf_set_int_value (LAST_ZOOMLEVEL, zoom);
 	}
 	itemview->priv->zoom = zoom;
-	
-	/* initially we pack the item list in the normal view pane,
-	   which is later changed in itemview_set_layout() */
-	gtk_container_add (GTK_CONTAINER (liferea_shell_lookup ("normalViewItems")), itemview->priv->itemListViewContainer);
 
-	/* 2. Prepare HTML rendering */
-	htmlview_init ();
+	/* 2. Set initial layout (because no node selected yet) */
+	itemview_set_layout (NODE_VIEW_MODE_WIDE);
 	
 	return itemview;
 }
